#ifndef IMCA_EPICS_UNIT_H
#define IMCA_EPICS_UNIT_H

#include <time.h>

#define failIf(context, testStmt) do { \
  if (testStmt) imcaFail(context, __FILE__, __LINE__); \
} while (0)
#define failIfIntNotEqual(context, valueStmt1, valueStmt2) do { \
  imcaFailIfIntNotEqual(context, valueStmt1, valueStmt2, __FILE__, \
    __LINE__); \
} while (0)
#define failIfDoubleNotEqual(context, valueStmt1, valueStmt2, delta) do { \
  imcaFailIfDoubleNotEqual(context, valueStmt1, valueStmt2, delta, __FILE__, \
    __LINE__); \
} while (0)
#define failIfDoubleArrayNotEqual(context, valueStmt1, valueStmt2, \
    valueSizeStmt, delta) do { \
  imcaFailIfDoubleArrayNotEqual(context, valueStmt1, valueStmt2, \
    valueSizeStmt, delta, __FILE__, __LINE__); \
} while (0)
#define failIfStringNotEqual(context, valueStmt1, valueStmt2) do { \
  imcaFailIfStringNotEqual(context, valueStmt1, valueStmt2, __FILE__, \
    __LINE__); \
} while (0)
#define runTest(context, testFuncName) do { \
  imcaStartingEpicsUnitTest(context); testFuncName(); \
} while (0)
#define runTests(context, testsFuncName) do { \
  imcaInitializeEpicsUnit(context); \
  imcaPrintEpicsUnitTestsStarting(); \
  imcaNoteNowAsEpicsUnitTestsStartTime(context); \
  testsFuncName(); \
  imcaNoteNowAsEpicsUnitTestsEndTime(context); \
  imcaPrintEpicsUnitTestsSummary(context); \
} while (0)

typedef struct EpicsUnitContext {
  int testThatFailed;
  int numTestsFailed;
  int testCounter;
  time_t testsStartTime;
  time_t testsEndTime;
} EpicsUnitContext;

extern void imcaFail(EpicsUnitContext *context, const char *fileName,
  int lineNumber);
extern void imcaFailIfIntNotEqual(EpicsUnitContext *context, int value1,
  int value2, const char *fileName, int lineNum);
extern void imcaFailIfDoubleNotEqual(EpicsUnitContext *context,
  double value1, double value2, double delta, const char *fileName,
  int lineNum);
extern void imcaFailIfDoubleArrayNotEqual(EpicsUnitContext *context,
  double values1[], double values2[], size_t valuesSize, double delta,
  const char *fileName, int lineNum);
extern void imcaFailIfStringNotEqual(EpicsUnitContext *context,
  const char *value1, const char *value2, const char *fileName, int lineNum);
extern void imcaNoteNowAsEpicsUnitTestsStartTime(EpicsUnitContext *context);
extern void imcaNoteNowAsEpicsUnitTestsEndTime(EpicsUnitContext *context);
extern void imcaPrintEpicsUnitTestProgress(void);
extern void imcaPrintEpicsUnitTestsStarting(void);
extern void imcaPrintEpicsUnitTestsSummary(EpicsUnitContext *context);
extern void imcaInitializeEpicsUnit(EpicsUnitContext *context);
extern void imcaStartingEpicsUnitTest(EpicsUnitContext *context);

#endif
