#ifndef IMCA_SMTP_CLIENT_H
#define IMCA_SMTP_CLIENT_H

#include <time.h>
#include <asynDriver.h>

/* Public interface */

#define IMCA_SMTP_DEFAULT_TIMEOUT 60.0
#define IMCA_SMTP_RESPONSE_SIZE 512
#define IMCA_SMTP_REQUEST_SIZE 512
#define IMCA_SMTP_MESSAGE_LINE_SIZE 998
#define IMCA_SMTP_HELO_HOST_KEY "IMCA_SMTP_HELO_HOST"
#define IMCA_SMTP_ENCODED_MESSAGE_SEND_BUFFER_SIZE 1024

typedef enum {
  ImcaEmailStatusSuccess,
  ImcaEmailStatusBufferTooSmall,
  ImcaEmailStatusEmptyFrom,
  ImcaEmailStatusEmptyTo,
  ImcaEmailStatusNetworkError,
  ImcaEmailStatusSmtpSuccess,
  ImcaEmailStatusSmtpDeferral,
  ImcaEmailStatusSmtpFailure,
  ImcaEmailStatusSmtpSyntaxError,
  ImcaEmailStatusSmtpHeloHostConfError,
  ImcaEmailStatusSmtpMailFromError,
  ImcaEmailStatusSmtpRcptToError
} ImcaEmailStatus;

extern ImcaEmailStatus imcaSendEmail(const char *asynSmtpClientPortName,
  const char *from, char *recipients[], int recipientsSize,
  const char *message, double smtpTimeout);
extern ImcaEmailStatus imcaCreateEmailMessage(char *dst, size_t dstSize,
  const char *from, char *to[], int toSize, char *cc[], int ccSize,
  const char *subject, const char *body);

/* Private interface */

extern int imcaPrintEmailDateField(char *dst, size_t dstSize,
  struct tm *dateAsTm);
extern int imcaPrintEmailFromField(char *dst, size_t dstSize,
  const char *from);
extern int imcaPrintEmailToField(char *dst, size_t dstSize, char *to[],
  int toSize);
extern int imcaPrintEmailCcField(char *dst, size_t dstSize, char *cc[],
  int ccSize);
extern int imcaPrintEmailSubjectField(char *dst, size_t dstSize,
  const char *subject);
extern int imcaPrintEmailBody(char *dst, size_t dstSize, const char *body);
extern int imcaPrintEmailDestinationAddressField(char *dst, size_t dstSize,
  const char *dieldName, char *addresses[], int addressesSize);

extern ImcaEmailStatus imcaSendEmailUsingAsynUser(asynUser *smtpClient,
  const char *from, char *recipients[], int recipientsSize,
  const char *message, double smtpTimeout);
extern ImcaEmailStatus imcaWaitForSmtpServerGreeting(asynUser *client,
  double timeout);
extern ImcaEmailStatus imcaSendSmtpHeloRequest(asynUser *client,
  double timeout);
extern ImcaEmailStatus imcaSendSmtpMailRequest(asynUser *client,
  double timeout, const char *from);
extern ImcaEmailStatus imcaSendSmtpRcptRequests(asynUser *client,
  double timeout, char *recipients[], int recipientsSize);
extern ImcaEmailStatus imcaSendSmtpRcptRequest(asynUser *client,
  double timeout, const char *recipient);
extern ImcaEmailStatus imcaSendSmtpDataRequest(asynUser *client,
  double timeout);
extern ImcaEmailStatus imcaSendSmtpEncodedMessage(asynUser *client,
  double timeout, const char *message);
extern ImcaEmailStatus imcaSendSmtpQuitRequest(asynUser *client,
  double timeout);
extern ImcaEmailStatus imcaReadSmtpResponse(asynUser *client, double timeout);
extern ImcaEmailStatus imcaSendSmtpRequestAndReadResponse(asynUser *client,
  double timeout, const char *request);
extern int imcaIsLastLineOfSmtpResponse(const char *response);
extern ImcaEmailStatus imcaGetStatusFromSmtpResponse(const char *response);
extern char *imcaGetSmtpHeloHost(void);

#endif
